/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DISPATCH_HH
#define DISPATCH_HH

#include <pthread.h>
#include <glib.h>

class Dp;
/*template<class C, typename Ret>
class Dp0;
template<class C, typename Ret, typename P1>
class Dp1;
template<class C, typename Ret, typename P1, typename P2>
class Dp2;
template<class C, typename Ret, typename P1, typename P2, typename P3>
class Dp3;*/

class Dispatch
{
public:
	~Dispatch();

	static Dispatch *getInstance();

        static void	*threadFunc( void *arg );	
	
	void		addToQueue	( Dp *dp );
	void		thread();

private:
	GAsyncQueue *queueList;

	Dispatch();

	static Dispatch	*dis;

	pthread_t 	dpThread;
	//pthread_mutex_t	dpLock;
	//sem_t		dpSem;
		
};

class Dp
{
public:
	Dp(){};
	virtual ~Dp(){};

	virtual void	run()	const	=	0;
};

template<class C, typename Ret>
class Dp0 : public Dp
{
public:
	Dp0( C *c,	Ret (C::*func)() )
	{
        	_c	=	c;
		_func	=	func;
	};

	~Dp0(){};

	inline void	run()	const
	{
        	(*_c.*_func)();
	};

private:
  	C	*_c;
	Ret	( C::*_func )();
};

template<class C, typename Ret, typename P1>
class Dp1 : public Dp
{
public:
	Dp1( C *c,	Ret (C::*func)(P1),	P1 par1)
	{
        	_c	=	c;
		_func	=	func;
		_par1	=	par1;
	};

	~Dp1(){};

	inline void	run()	const
	{
        	(*_c.*_func)(_par1);
	};

private:
  	C	*_c;
	Ret	( C::*_func )( P1 );
	P1	_par1;
};

template<class C, typename Ret, typename P1, typename P2>
class Dp2 : public Dp
{
public:
	Dp2( C *c,	Ret (C::*func)(P1, P2),	P1 par1,	P2 par2 )
	{
        	_c	=	c;
		_func	=	func;
		_par1	=	par1;
		_par2	=	par2;	
	};

	~Dp2(){};

	inline void	run()	const
	{
        	(*_c.*_func)(_par1,	_par2);
	};

private:
  	C	*_c;
	Ret	( C::*_func )( P1, P2 );
	P1	_par1;
	P2	_par2;
};

template<class C, typename Ret, typename P1, typename P2, typename P3>
class Dp3 : public Dp
{
public:
	Dp3( C *c,	Ret (C::*func)(P1, P2, P3),	P1 par1,	P2 par2,	P3 par3 )
	{
        	_c	=	c;
		_func	=	func;
		_par1	=	par1;
		_par2	=	par2;
		_par3	=	par3;	
	};

	~Dp3(){};

	inline void	run()	const
	{
        	(*_c.*_func)(_par1,	_par2,	_par3);
	};

private:
  	C	*_c;
	Ret	( C::*_func )( P1, P2, P3 );
	P1	_par1;
	P2	_par2;
	P3	_par3;
};

#else
class Dispatch;
class Dp;

template<class C, typename Ret>
class Dp0;
template<class C, typename Ret, typename P1>
class Dp1;
template<class C, typename Ret, typename P1, typename P2>
class Dp2;
template<class C, typename Ret, typename P1, typename P2, typename P3>
class Dp3;
#endif
