/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//#include <glib/gstdio.h>
#include <gdk/gdkkeysyms.h>

#if ENABLE_ICONVIEW
	#include "gtkiconview.h"
#else
	#include <gtk/gtkiconview.h>
#endif

#include "fileview.h"

using namespace std;

//-------------------------------------------------------------------------
// constructor
FileView::FileView( string path )
{
	mw	=	MainWindow::getInstance();

	GladeXML *xml	=	mw->getXml();
	
	menu		=	GTK_MENU( glade_xml_get_widget( xml,	"menu5" ) );
	
	copyItem		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"copy" ) );
	cutItem			=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"cut" ) );
	pasteItem		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"paste" ) );
	deleteItem		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"delete" ) );
	newFolder		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"new_folder" ) );
	newFile			=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"new_file" ) );
	newLink			=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"new_symbolic_link_to" ) );
	properties		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"properties" ) );
	newTabItem 		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"open_new_tab2" ) );
	renameItem		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"rename" ) );
	openItem		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"open_with_1" ) );
	bookmarkItem		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"add_bookmark_here1" ) );
	newSubmenu		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"new1" ) );
	actionsSubmenu		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"actions1" ) );
	openWithSubmenu		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"open_with" ) );
	
	dnd_menu 		= GTK_MENU( glade_xml_get_widget( xml,	"dnd_menu" ) );
	
	dnd_move		= GTK_MENU_ITEM(glade_xml_get_widget( xml,	"dnd_move" ) );
	dnd_copy		= GTK_MENU_ITEM(glade_xml_get_widget( xml,	"dnd_copy" ) );
	dnd_symlink		= GTK_MENU_ITEM(glade_xml_get_widget( xml,	"dnd_symlink" ) );
	
	GtkAccelGroup *group	=	mw->getAccelGroup();
	gtk_menu_set_accel_group( menu,	group );
	
	gtk_widget_add_accelerator( GTK_WIDGET( copyItem ),
				  				"activate",
				  				group,
				  				GDK_c,
				  				GDK_CONTROL_MASK,
				  				GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator( GTK_WIDGET( cutItem ),
				  				"activate",
				  				group,
				  				GDK_x,
				  				GDK_CONTROL_MASK,
				  				GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator( GTK_WIDGET( pasteItem ),
				  				"activate",
				  				group,
				  				GDK_v,
				  				GDK_CONTROL_MASK,
				  				GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator( GTK_WIDGET( deleteItem ),
				  				"activate",
				  				group,
				  				GDK_Delete,
				  				GDK_CONTROL_MASK,
				  				GTK_ACCEL_VISIBLE);
	
	gtk_widget_set_sensitive( GTK_WIDGET( pasteItem ),	false );
	
	window	=	gtk_scrolled_window_new( NULL,	NULL );
	
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( window ), 
									GTK_POLICY_AUTOMATIC,	GTK_POLICY_AUTOMATIC );
	
	fam	=	new FAMConnection;
	fr	=	new FAMRequest;
	
	if( FAMOpen2( fam,	"gbrowser" ) < 0 )
	{
		g_free( fam );
		fam	=	NULL;
	}
	
	if( gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM( mw->iconView ) ) ) 
		this->buildIconView();
	else if( gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM( mw->listView ) ) ) 
		this->buildListView();
	else 
		this->buildImageView();
	
	currentDir	=	path;
	showHidden	=	mw->getHidden();
	gesture		=	false;
	showMenu	=	true;
	
	this->createSignalHandlers();
	this->scanDirectory( currentDir );
}

//-------------------------------------------------------------------------
// destructor
FileView::~FileView()
{
	FAMClose( fam );
	g_free( fr );
}

//-------------------------------------------------------------------------
// createSignalHandlers method
void FileView::createSignalHandlers()
{		
	if( type != IMAGE_VIEW )
	{
		GladeXML *xml	=	mw->getXml();
		menu		=	GTK_MENU( glade_xml_get_widget( xml,	"menu5" ) );

		GdkDragAction	action		=	GdkDragAction( GDK_ACTION_COPY		|	GDK_ACTION_ASK );
		GdkModifierType	modifier	=	GdkModifierType( GDK_BUTTON1_MASK	|	GDK_BUTTON3_MASK );
		
		gtk_drag_dest_set( window,	GTK_DEST_DEFAULT_ALL,	target,	3,	action );
		
		if( type	==	ICON_VIEW)
		{
			gtk_icon_view_enable_model_drag_source(	GTK_ICON_VIEW( view ),	modifier,	target,	3,	action );
			
			iconEnterCallback.connect( G_OBJECT( view ),		"item-activated",
								this,	&FileView::iconEnterFolder,	NULL );
			showMenuCallback.connect( G_OBJECT( view ),	"button-press-event",
								this,	&FileView::showPopupMenu,	NULL );
		}
		
		else if( type	==	LIST_VIEW )
		{
			gtk_tree_view_enable_model_drag_source( GTK_TREE_VIEW(view),	modifier,	target,	3,	action );
			
			showMenuCallback.connect( G_OBJECT( view ),	"button-press-event",
								this,	&FileView::showPopupMenu,	NULL );
			listEnterCallback.connect( G_OBJECT( view ),		"row-activated",
								this,	&FileView::listEnterFolder,	NULL );
		}

		buttonReleaseCallback.connect(		G_OBJECT( view ),		"button-release-event",	
								this,	&FileView::buttonReleased,		NULL );
		dragReceiveCallback.connect(		G_OBJECT( window ),	"drag-data-received", 
								this,	&FileView::dragReceived,		NULL );
		dragStartCallback.connect(		G_OBJECT( view ),		"drag-begin",
								this,	&FileView::dragStart,			NULL );
		dragEndCallback.connect_after(		G_OBJECT( view ),	"drag-end",	
								this,	&FileView::dragEnd,			NULL );
		getDataCallback.connect(		G_OBJECT( view ),		"drag-data-get",
								this,	&FileView::getDragData,			NULL );
		copyCallback.connect(			G_OBJECT( copyItem ),		"activate",	
								this,	&FileView::copyClicked,			NULL );
		cutCallback.connect(			G_OBJECT( cutItem ),		"activate",	
								this,	&FileView::cutClicked,			NULL );
		pasteCallback.connect(			G_OBJECT( pasteItem ),		"activate",	
								this,	&FileView::pasteClicked,		NULL );
		deleteCallback.connect(			G_OBJECT( deleteItem ),		"activate",
								this,	&FileView::deleteClicked,		NULL );
		newFolderCallback.connect(		G_OBJECT( newFolder ),		"button-release-event",
								this,	&FileView::newFolderClicked,		NULL );
		newFileCallback.connect(		G_OBJECT( newFile ),		"button-release-event",
								this,	&FileView::newFileClicked,		NULL );
		newLinkCallback.connect(		G_OBJECT( newLink ),		"button-release-event",
								this,	&FileView::newLinkClicked,		NULL );
		renameCallback.connect(			G_OBJECT( renameItem ),		"activate", 
								this,	&FileView::renameClicked,		NULL );
		dndCallback.connect(			G_OBJECT( dnd_move ),		"activate", 
								this,	&FileView::dndMenuClicked,		NULL );
		dndCallback.connect(			G_OBJECT( dnd_copy ),		"activate", 
								this,	&FileView::dndMenuClicked,		NULL );
		dndCallback.connect(			G_OBJECT( dnd_symlink ),	"activate", 
								this,	&FileView::dndMenuClicked,		NULL );
		openWithCallback.connect(		G_OBJECT( openItem ),		"button-release-event",
								this,	&FileView::openWithClicked,		NULL );
		addBookmarkCallback.connect(		G_OBJECT( bookmarkItem ),	"button-release-event",
								this,	&FileView::addBookmarkClicked,		NULL );	

	}

	else
	{ 
		changeImageCallback.connect(	G_OBJECT(prevButton),		"clicked", 
						this,	&FileView::changeImageClicked,	NULL );
		changeImageCallback.connect(	G_OBJECT(nextButton),		"clicked", 
						this,	&FileView::changeImageClicked,	NULL );
		imageClickedCallback.connect(	G_OBJECT(view),			"button-press-event",
						this,	&FileView::imageClicked,	NULL );
		imageSelectionChanged.connect_after( G_OBJECT( thumbnailView ),	"selection-changed",
						this,	&FileView::imageChanged,	NULL );
		rotateImageCallback.connect(	G_OBJECT( rotateLeft ),		"clicked",
						this,	&FileView::rotateImageClicked,	NULL );
		rotateImageCallback.connect(	G_OBJECT( rotateRight ),	"clicked",
						this,	&FileView::rotateImageClicked,	NULL );
	}
   		fileInfoCallback.connect(	G_OBJECT( properties ),		"activate", 
										this,	&FileView::fileInfoClicked,	NULL );
}

//-------------------------------------------------------------------------
// buildIconView method
void FileView::buildIconView()
{
	fileStore	=	gtk_list_store_new( 5,	GDK_TYPE_PIXBUF,	G_TYPE_STRING,	
										G_TYPE_STRING,	G_TYPE_STRING,	G_TYPE_BOOLEAN );
	view 		=	gtk_icon_view_new_with_model( GTK_TREE_MODEL( fileStore ) );
	
	gtk_icon_view_set_selection_mode( GTK_ICON_VIEW( view ),	GTK_SELECTION_MULTIPLE );
	gtk_icon_view_set_item_width( GTK_ICON_VIEW( view ),	100 );

	gtk_icon_view_set_text_column( GTK_ICON_VIEW( view ),	COLUMN_NAME );
	gtk_icon_view_set_pixbuf_column( GTK_ICON_VIEW( view ),	COLUMN_ICON );
	
	gtk_container_add( GTK_CONTAINER( window ),	view );
	
	type	=	ICON_VIEW;
}

//-------------------------------------------------------------------------
// buildListView method
void FileView::buildListView()
{
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
	
	fileStore	=	gtk_list_store_new(9,	GDK_TYPE_PIXBUF,	G_TYPE_STRING,
									G_TYPE_STRING,	G_TYPE_STRING,	
									G_TYPE_BOOLEAN,	G_TYPE_STRING,
						  			G_TYPE_STRING,	G_TYPE_STRING,
						  			G_TYPE_STRING);
	view 		=	gtk_tree_view_new_with_model( GTK_TREE_MODEL( fileStore ) );
	selection	=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
	gtk_tree_selection_set_mode( selection,	GTK_SELECTION_MULTIPLE );

  	col	=	gtk_tree_view_column_new();
  	gtk_tree_view_column_set_title( col,	"" );
  	gtk_tree_view_append_column( GTK_TREE_VIEW( view ),	col );
  	renderer	=	gtk_cell_renderer_pixbuf_new();
  	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
  	gtk_tree_view_column_add_attribute( col,	renderer,	"pixbuf",	COLUMN_ICON );
	
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Name" );
  	gtk_tree_view_append_column( GTK_TREE_VIEW( view ),	col );
  	renderer	=	gtk_cell_renderer_text_new();
  	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
  	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	COLUMN_NAME );
	
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Size" );
  	gtk_tree_view_append_column( GTK_TREE_VIEW( view ),	col );
  	renderer	=	gtk_cell_renderer_text_new();
  	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
  	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	COLUMN_SIZE );
	
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Owner" );
	gtk_tree_view_append_column( GTK_TREE_VIEW( view ),	col );
	renderer	=	gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	COLUMN_OWNER );
	
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Group" );
	gtk_tree_view_append_column( GTK_TREE_VIEW( view ),	col );
	renderer	=	gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	COLUMN_GROUP );
	
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Last Access" );
	gtk_tree_view_append_column( GTK_TREE_VIEW( view ),	col );
	renderer	=	gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	COLUMN_ACCESS );

	gtk_container_add( GTK_CONTAINER( window ),	view );
	
	type	=	LIST_VIEW;
}

//-------------------------------------------------------------------------
// buildImageView method
void FileView::buildImageView()
{
	string executable	=	mw->getPathToExecutable();
	string dataPath	=	mw->getGladeFile( "imageview.glade" );
	
	GladeXML	*x	=	glade_xml_new( dataPath.c_str(),	NULL,	NULL );
	GtkWidget	*w	=	glade_xml_get_widget( x,	"window1" );
	
	view	=	gtk_viewport_new( NULL,		NULL );
	
	GtkWidget 	*box	=	glade_xml_get_widget( x,	"vbox1" );
	GtkWidget	*win	=	glade_xml_get_widget( x,	"scrolledwindow2" );
	
	image			=	GTK_IMAGE(  glade_xml_get_widget( x,	"image1" ) );
	prevButton		=	GTK_BUTTON( glade_xml_get_widget( x,	"prevImage" ) );
	nextButton		=	GTK_BUTTON( glade_xml_get_widget( x,	"nextImage" ) );
	rotateLeft		=	GTK_BUTTON( glade_xml_get_widget( x,	"rotateLeft" ) );
	rotateRight		=	GTK_BUTTON( glade_xml_get_widget( x,	"rotateRight" ) );
	
	imageNumStatus		=	GTK_STATUSBAR( glade_xml_get_widget( x,	"image_num_status" ) );
	
	imageNameEntry	=	GTK_ENTRY( glade_xml_get_widget( x,	"image_name_entry" ) );

    	menu		=	GTK_MENU( glade_xml_get_widget( x,	"menu1" ) );

	properties	=	GTK_MENU_ITEM( glade_xml_get_widget( x,	"properties" ) );
	
	thumbnailStore		=	gtk_list_store_new( 4,	GDK_TYPE_PIXBUF,	G_TYPE_STRING,	G_TYPE_STRING,	G_TYPE_INT);
	thumbnailView		=	gtk_icon_view_new_with_model( GTK_TREE_MODEL( thumbnailStore ) );

	gtk_icon_view_set_pixbuf_column( GTK_ICON_VIEW( thumbnailView ),	COLUMN_ICON );
	gtk_icon_view_set_column_spacing( GTK_ICON_VIEW( thumbnailView),	15 );
	
	gtk_widget_ref( box );
	gtk_container_remove( GTK_CONTAINER( w ),	box );
	gtk_widget_destroy( w );
	
	gtk_container_add( GTK_CONTAINER( win ),			thumbnailView );
	gtk_container_add( GTK_CONTAINER( view ),		box );
	gtk_container_add( GTK_CONTAINER( window ),	view );
	
	gtk_widget_show_all( view );
	
	step	=	0;
	
	type	=	IMAGE_VIEW;
}

//-------------------------------------------------------------------------
// showPopupMenu method
gboolean FileView::showPopupMenu( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{	
	if( button->button == 2 )
	{
		gesture	=	true;
		x_axis	=	button->x;
		y_axis	=	button->y;
		//TODO: should we draw something here?
	}

	return false;
}

//-------------------------------------------------------------------------
// enterFolder method
void FileView::enterFolder( GtkTreePath *path )
{
	GtkTreeIter iter;
	
	gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&iter,	path );

	char *d;
	gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&iter,
                        COLUMN_PATH,	&d,	-1 );
	
	if( !g_file_test( d,	G_FILE_TEST_IS_DIR ) )
	{ 
		this->openFile( d );
		return;
	}

	prevDirs.push_front( currentDir );

	if( type	!=	IMAGE_VIEW ) 
		this->scanDirectory( d );
	else 
		this->scanImages( d );
		
	g_free( d );
	
	mw->setAllowPrev();
	mw->setAllowNext();
}

//-------------------------------------------------------------------------
// clearTreeViews method
void FileView::clearTreeViews()
{
	gtk_list_store_clear( fileStore );
}

//-------------------------------------------------------------------------
// removeEntries method
void FileView::removeEntries( GtkListStore  *store, list<string> data )
{
	if( store == NULL )
		store = fileStore;

	GtkTreeModel *model	=	GTK_TREE_MODEL( store );

	GtkTreeIter i;
	list<string>::iterator iter;
	vector<GtkTreeRowReference *> ref;
	const char *path;

	gtk_tree_model_get_iter_first( model,	&i );
	
	while( gtk_list_store_iter_is_valid( store,	&i ) )
	{
		gtk_tree_model_get( GTK_TREE_MODEL( store ),	&i,	COLUMN_PATH,	&path,	-1 );
		
		iter	=	find( data.begin(),	data.end(),	path );

		if( iter	!=	data.end() ) 
			ref.push_back( gtk_tree_row_reference_new( model, 
							gtk_tree_model_get_path( model,	&i ) ) );

		gtk_tree_model_iter_next( model,	&i );
		
		if( !gtk_list_store_iter_is_valid( store,	&i ) ) break;
	}
	
	if( ref.size()	!=	0 )
	{
		for( vector<GtkTreeRowReference*>::iterator it	=	ref.begin(); 
													it	!=	ref.end(); 
													it++)
		{
			gtk_tree_model_get_iter( model,	&i,	gtk_tree_row_reference_get_path( (*it) ) );
			gtk_list_store_remove(	store,	&i );
		}
	}
}

//-------------------------------------------------------------------------
// createPasteThread method
void *FileView::createPasteThread( void *arg )
{
	PasteFunc *func	=	( PasteFunc* )arg;
	func->pasteThread();
}

//-------------------------------------------------------------------------
// createDeleteThread method
void *FileView::createDeleteThread( void *arg )
{
	PasteFunc *func	=	( PasteFunc* )arg;
	func->deleteThread();
}

//-------------------------------------------------------------------------
// scanDirectory method
void FileView::scanDirectory( string path )
{
	typedef Dp1<MainWindow, void, string> dp1;
	
	Dispatch 	*dp	=	Dispatch::getInstance();
	dp1 		*func;

	if( !Util::checkReadPermissions( path ) )
	{
		mw->updateStatusbar( "You don't have read permissions for " + path );
		return;
	}
	
	if( tag > 0 )
	{
		gdk_input_remove( tag ); 
		FAMCancelMonitor( fam,	fr );
	}
		
	FAMMonitorDirectory( fam,	path.c_str(),	fr,	(void*)this );

	tag	=	gdk_input_add( FAMCONNECTION_GETFD( fam ),	GDK_INPUT_READ,	FileView::readFamInput,	fam );

	func	=	new dp1( mw,	&MainWindow::updateStatusbar,	"Scanning directory.." );
	dp->addToQueue( func );
	
	typedef Dp1<Sidebar, void, string> dp1_bar;
	dp1_bar *barFunc	=	new dp1_bar( Sidebar::getInstance(),	&Sidebar::treeViewSetPath,	path );
	dp->addToQueue( barFunc );

	mw->setPath( path );
	
	this->clearTreeViews();

	const char *name;

	GDir *dir	=	g_dir_open(path.c_str(), 0, NULL);
	name		=	g_dir_read_name(dir);
	
	g_dir_rewind( dir );
	
	currentDir	=	path;

	int hiddenCount	=	0;
	int itemCount		=	0;
	
	list<string>dirList,	fileList;
	
	while( name	!=	NULL )
	{
		name	=	g_dir_read_name( dir );
		if(name == NULL) 
			break;
		
		string	fullPath; 

		if( g_str_has_suffix( currentDir.c_str(),	"/") ) 
			fullPath	= currentDir	+	name;		
		else 
			fullPath	= currentDir	+	"/"	+	name;

		itemCount++;

		if( g_str_has_prefix( name,	"." ) )
		{
			hiddenCount++;

			if( !showHidden ) 
				continue;
		}
		
		if( g_file_test( fullPath.c_str(),	G_FILE_TEST_IS_DIR ) )
			dirList.push_back( fullPath );
		else
			fileList.push_back( fullPath );
	}
	
	g_dir_close( dir );
	
	fileList.sort();
	dirList.sort();
	
	list<string>::iterator it;
	
	for( it = dirList.begin();	it != dirList.end();	it++ )
	{
		typedef Dp3<FileView, void, string, string, bool> dp3;
		dp3 *func	=	new dp3( this,	&FileView::handleFile,	g_path_get_basename( (*it).c_str() ),	(*it),	false );

		Dispatch::getInstance()->addToQueue( func );
	}
	
	for( it = fileList.begin();	it != fileList.end();	it++ )
	{
		typedef Dp3<FileView, void, string, string, bool> dp3;
		dp3 *func	=	new dp3( this,	&FileView::handleFile,	g_path_get_basename( (*it).c_str() ),	(*it),	false );

		Dispatch::getInstance()->addToQueue( func );
	}

	string stat;

        if( !Util::checkWritePermissions( path ) ) 
		stat	=	"You don't have write permissions in this folder";
	else
		stat	=	"Scanning directory.. Done";

	mw->updateCounters( itemCount,	hiddenCount );

	func	=	new dp1( mw,	&MainWindow::updateStatusbar,	stat );
	dp->addToQueue( func );		
}

//-------------------------------------------------------------------------
// deleteClicked method
void FileView::deleteClicked( GtkWidget *widget,	gpointer data )
{
	const char *path;
	
	GList *selections;
	
	if( type	==	ICON_VIEW ) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	
	else if( type	==	LIST_VIEW )
	{
		selection	=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}
	
	GList *list = g_list_first( selections );
	GtkTreeIter i;
	
	if( pasteData.size()	>	0 ) 
		pasteData.clear();

	if( !list ) 
		return;
	
	while(1)
	{
		gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&i, 
								( GtkTreePath* )list->data );
		gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&i, 
							COLUMN_PATH,	&path,	-1 );
		
		pasteData.push_back( path );

		list	=	g_list_next( list );
		
		if( !list ) 
			break;
	}

	pasteData	=	checkFileListExist( pasteData );
	mw->setDeleteData( pasteData );

	PasteFunc *p	=	new	PasteFunc( this,	TYPE_DELETE );
	pthread_create( &pasteThread,	NULL,	&createDeleteThread,	( void* )p );
	g_list_free( list );
	g_list_free( selections );
}

//-------------------------------------------------------------------------
// copyClicked method
void FileView::copyClicked( GtkWidget *widget,	gpointer data )
{
	const char *path;

	GList *selections;
	
	if(type	==	ICON_VIEW) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if(type == LIST_VIEW)
	{
		selection	=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}
	
	GList *list	=	g_list_first( selections );
	GtkTreeIter i;
	
	if( pasteData.size()	>	0 )
		pasteData.clear();

	if( !list ) 
		return;
	
	while( 1 )
	{
		gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&i, 
								( GtkTreePath* )list->data );
		gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&i,	
							COLUMN_PATH,	&path,	-1 );
		
		pasteData.push_back( path );
		
		list	=	g_list_next( list );
		
		if(!list) 
			break;
	}

	mw->setPasteData( pasteData );

	pasteType	=	TYPE_COPY;
	gtk_widget_set_sensitive( GTK_WIDGET( pasteItem ),	true );
	g_list_free( list );
	g_list_free( selections );
}

//-------------------------------------------------------------------------
// cutClicked method
void FileView::cutClicked( GtkWidget *widget,	gpointer data )
{
	const char *path;

	GList *selections;
	
	if( type	==	ICON_VIEW ) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if( type	==	LIST_VIEW )
	{
		selection	=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}
	
	GList *list	=	g_list_first( selections );
	GtkTreeIter i;
	
	if( pasteData.size()	>	0 ) 
		pasteData.clear();

	if( !list ) 
		return;
		
	while( 1 )
	{
		gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&i, 
								( GtkTreePath* )list->data );
		gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&i, 
							COLUMN_PATH,	&path,	-1 );
		
		pasteData.push_back( path );
		
		list	=	g_list_next( list );
		
		if(!list) 
			break;
	}

	mw->setPasteData( pasteData );

	pasteType	=	TYPE_CUT;
	gtk_widget_set_sensitive( GTK_WIDGET( pasteItem ),	true );
	g_list_free( list );
	g_list_free( selections );
}

//-------------------------------------------------------------------------
// pasteClicked method
void FileView::pasteClicked(GtkWidget *widget, gpointer data)
{
	if( !Util::checkWritePermissions( currentDir ) )
	{
		Dialog::noPermissions( currentDir );
		return;
	}
		
	pasteData	=	checkFileListExist( mw->getPasteData() );
	mw->setPasteData( pasteData );
	
	if( pasteType	!=	TYPE_DELETE ) 
		gtk_widget_set_sensitive( GTK_WIDGET( pasteItem ),	false );
	
	PasteFunc *p	=	new	PasteFunc( this,	pasteType );
	pthread_create( &pasteThread,	NULL,	&createPasteThread,	p );
}

//-------------------------------------------------------------------------
// startScanning method
void FileView::startScanning( string path )
{
	prevDirs.push_front( currentDir );

	if( type	!=	IMAGE_VIEW ) 
		this->scanDirectory( path );
	else 
		this->scanImages( path );
}

//-------------------------------------------------------------------------
// handlePrev method
void FileView::handlePrev( string path )
{
	nextDirs.push_front( currentDir );
	prevDirs.pop_front();
	
	if( type	!=	IMAGE_VIEW ) 
		this->scanDirectory( path );
	else 
		this->scanImages( path );
}

//-------------------------------------------------------------------------
// handleNext method
void FileView::handleNext( string path )
{
	prevDirs.push_front( currentDir );
	nextDirs.pop_front();
	
	if( type	!=	IMAGE_VIEW ) 
		this->scanDirectory( path );
	else 
		this->scanImages( path );
}

//-------------------------------------------------------------------------
// handleParent method
void FileView::handleParent( string path )
{
	prevDirs.push_front( currentDir );
	
	if( type	!=	IMAGE_VIEW ) 
		this->scanDirectory( path );
	else 
		this->scanImages( path );
}

//-------------------------------------------------------------------------
// openFile method
void FileView::openFile( const string& path )
{
	string mime;
	const char *tmp =	xdg_mime_get_mime_type_for_file( path.c_str() );
	
	if( tmp	==	NULL	&&	g_str_has_suffix( path.c_str(),	".avi" ) ) 
		mime	=	"video/x-msvideo";
	else 
		mime	=	tmp;

	string app	=	this->getDefaultRunAction( mime );
	if( app.length()	==	0 )
	{ 
		app	=	Dialog::setDefaultRunAction( mime,	g_get_home_dir() );
		if( app.length()	==	0 ) 
			return;
	}
	
	string cmd	=	app	+	(string)" \""	+	path	+	(string)"\"";
	
	if( this->needsTerminal( mime,	app ) )
	{
		mw->openNewTerminalWithCommand( cmd );
		return;
	}

	Util::exec( cmd );
}

//-------------------------------------------------------------------------
// newFolderClicked method
gboolean FileView::newFolderClicked( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{
	string command	=	"mkdir ";
	string path;
	string name	=	Dialog::createNewFuncDialog( "Directory name: ",	"Create directory", 
												"",	currentDir,	true );

	if( name.length()	==	0 ) 
		return false;
	
	if( !g_str_has_prefix( name.c_str(),	"/") ) 
		path = currentDir + "/" + name;
	else 
		path = name;

	bool exists	=	g_file_test( path.c_str(),	G_FILE_TEST_EXISTS );
	bool check	=	true;

	if( exists ) 
		check	=	Dialog::fileExists( g_path_get_basename( name.c_str() ) );
	
	if( !check ) 
		return false;
	
	if( exists )
	{
		string cmd	=	"rm -rf "	+	path;
		system( cmd.c_str() );
	}
		
	command	+=	"\""	+	path	+	(string)"\"";

	system( command.c_str() );
	
	return false;
}

//-------------------------------------------------------------------------
// newFileClicked method
gboolean FileView::newFileClicked( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{
	string command	=	"touch ";
	string path;
	string name		=	Dialog::createNewFuncDialog( "Filename: ",	"Create file", 
													"",	currentDir,	true );

	if( name.length()	==	0 ) 
		return false;
	
	if( !g_str_has_prefix( name.c_str(),	"/" ) ) 
		path	=	"\""	+	currentDir	+	"/"	+	name	+	(string)"\"";
	else 
		path	=	"\""	+	name	+ "\"";

	bool exists	=	g_file_test( path.c_str(),	G_FILE_TEST_EXISTS );
	bool check	=	true;

	if( exists ) check	=	Dialog::fileExists( g_path_get_basename( name.c_str() ) );
		
	if( !check ) 
		return false;
	
	if( exists )
	{		
		string cmd = "rm -rf " + path;
		system(cmd.c_str());
	}

	command	+=	path;

	system( command.c_str() );

	int len	=	path.length();
	string realPath	=	path.substr( 1,	len	-	2 );

	//if( !exists ) 
	//	this->handleFile( g_path_get_basename( realPath.c_str() ),	realPath,	true );
	
	return false;
}

//-------------------------------------------------------------------------
// newLinkClicked method
gboolean FileView::newLinkClicked( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{	
	string command	=	"ln -sf ";
	string input	= "\""	+	currentDir	+	( string )"/";
	string name		= Dialog::createNewFuncDialog( "Name of symbolic link: ", 
												 	"Create symbolic link", 
													"",	currentDir,	true)	+	(string)"\"";

	if( name.length()	==	0 ) 
		return false;
	
	string label	=	"Link "	+	(string)"\""	+	name	+	(string)" to: ";
	string output	=	"\""	+	currentDir		+	(string)"/";
	string n		=	Dialog::createNewFuncDialog( label,	"Create symbolic link", 
													"",	currentDir,	true)	+	(string)"\"";

	if( n.length()	==	0 ) 
		return false;
	
	if( g_str_has_prefix( name.c_str(),	"/" ) ) 
		input	=	"\""	+	name;
	else 
		input	+=	name;

	if( g_str_has_prefix( n.c_str(),	"/" ) ) 
		output	=	"\""	+	n;
	else 
		output	+=	n;

	int len			=	input.length();
	string realPath	=	input.substr( 1,	len	-	1 );

	bool exists	=	g_file_test( realPath.c_str(),	G_FILE_TEST_EXISTS );
	bool check	=	true;

	if( exists ) 
		check	=	Dialog::fileExists( g_path_get_basename( realPath.c_str() ) );
	
	if( !check ) 
		return false;
	
	if( exists )
	{ 
		string cmd	=	"rm -rf "	+	realPath;
		system( cmd.c_str() );
	}

	command	+=	output	+	" "	+	input;
	system( command.c_str() );
	
	return false;
}

//-------------------------------------------------------------------------
// fileInfoClicked method
void FileView::fileInfoClicked( GtkWidget *widget,	gpointer data )
{
	const char *path;
	
	GList *selections;

	if( type	==	ICON_VIEW ) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if( type	==	LIST_VIEW )
	{
		selection	=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}
	else
	{
		GtkTreeIter iter;
		
		GList *sel	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( thumbnailView ) );
		sel	=	g_list_first( sel );
		
		gtk_tree_model_get_iter( GTK_TREE_MODEL( thumbnailStore ),	&iter,	(GtkTreePath*)sel->data );
		
		g_list_free( sel );
		
		gtk_tree_model_get( GTK_TREE_MODEL( thumbnailStore ),	&iter,	COL_PATH,	&path,	-1 );
		FileInfo::getInstance( this )->setEntry( path );
		return;
	}

	GList *list	=	g_list_first( selections );
	GtkTreeIter i;

	if( !list ) 
		return;
	
	while(1)
	{
		gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&i, 
								( GtkTreePath* )list->data );
		gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&i, 
							COLUMN_PATH,	&path,	-1 );

		FileInfo::getInstance( this )->setEntry( path );
		
		list	=	g_list_next( list );
		if( !list ) 
			break;
	}
	g_list_free( list );
	g_list_free( selections );
}

//-------------------------------------------------------------------------
// getDefaultRunAction method
string FileView::getDefaultRunAction( const string& mime )
{
	string actionsFile	=	g_get_home_dir()	+	(string)"/.gbrowser/actions";
	
	GKeyFile* actions	=	g_key_file_new();
	g_key_file_load_from_file( actions,	actionsFile.c_str(),	G_KEY_FILE_NONE,	NULL );

	if( !g_key_file_has_group( actions,	mime.c_str() ) ) 
		return "";

	string runAction	=	g_key_file_get_string( actions,	mime.c_str(),	"Default",	NULL );

	g_key_file_free( actions );
	
	return runAction;
}

//-------------------------------------------------------------------------
// renameClicked method
void FileView::renameClicked( GtkWidget *widget,	gpointer data )
{
	const char *path;
	GtkTreeIter i;
	GList *selections;

	if( type	==	ICON_VIEW ) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if( type	==	LIST_VIEW )
	{
		selection	=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}
	
	GList *list	=	g_list_first( selections );

	if( !list ) 
		return;
		
	while(1)
	{
		gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&i, 
								( GtkTreePath* )list->data );
		gtk_tree_model_get( GTK_TREE_MODEL(fileStore),	&i, 
							COLUMN_PATH,	&path,	-1 );
		
		string lab	=	"Rename " +	(string)g_path_get_basename( path );
		string out	= 	Dialog::createNewFuncDialog( lab,	"Rename",	"",	currentDir,	true );
		
		if( out.length()	>	0 )
		{
			rename( path, out.c_str() );
			gtk_list_store_set( fileStore,	&i,	
								COLUMN_NAME,	g_path_get_basename( out.c_str() ),
								COLUMN_PATH,	out.c_str(),	-1 );
		}
		
		list	=	g_list_next( list );
		if( !list ) 
			break;
	}
	
	g_list_free( list );
	g_list_free( selections );
}

//-------------------------------------------------------------------------
// setView method
void FileView::setView( GtkRadioMenuItem *item )
{
	gtk_widget_ref( view );
	gtk_widget_destroy( view );
	
	if( item	==	mw->iconView ) 
		this->buildIconView();
	
	else if( item	==	mw->listView ) 
		this->buildListView();
	
	else 
		this->buildImageView();
	
	gtk_widget_show_all( window );
	
	this->createSignalHandlers();
	
	if( type != IMAGE_VIEW ) 
		this->scanDirectory(currentDir);
	
	else 
		this->scanImages(currentDir);
}

//-------------------------------------------------------------------------
// scanImages method
void FileView::scanImages( string path )
{
	const char *name;
	GtkTreeIter iter;
	
	GDir *dir	=	g_dir_open( path.c_str(),	0,	NULL );
	name		=	g_dir_read_name( dir );
	
	g_dir_rewind( dir );
	
	currentDir	=	path;
		
	mw->updateStatusbar( "Scanning for images..." );

	imageTotal		=	0;
	imageAngleChanged	=	false;

	gtk_list_store_clear( thumbnailStore );
	
	while( name != NULL )
	{
		name	=	g_dir_read_name( dir );
		
		if( name	==	NULL ) 
			break;
		
		string fullPath; 	
		
		if( g_str_has_suffix( currentDir.c_str(),	"/" ) ) 
			fullPath	=	currentDir	+	name;		
		else 
			fullPath	=	currentDir	+	"/"	+	name;

		const char *mime	=	xdg_mime_get_mime_type_for_file( fullPath.c_str() );
		
		if( mime	!=	NULL	&&	g_strrstr( mime,	"image" ) ) 
		{			
			gtk_list_store_append( thumbnailStore,	&iter );
			
			GdkPixbuf *buf	= Thumbnail::getInstance()->getThumbnailFromDatabase( thumbnailStore,	iter,	fullPath,	80 );
			
                        imageTotal++;

			gtk_list_store_set( thumbnailStore,	&iter,	
						COL_ICON,	buf,	
						COL_NAME,	g_path_get_basename( fullPath.c_str() ),
						COL_PATH,	fullPath.c_str(),
						COL_POS,	imageTotal,
						-1 );	
		}
		
	}
	
	if( imageTotal	==	0 )
	{
		gtk_entry_set_text( imageNameEntry,	"No images in this folder" );
		mw->updateStatusbar( "No images in this folder" );
	}
	
	else
	{				
		imageCount	=	1;
		
		gtk_icon_view_set_columns( GTK_ICON_VIEW( thumbnailView ),	imageTotal );
		gtk_tree_model_get_iter_first( GTK_TREE_MODEL( thumbnailStore ),	&iter );

		const char *path;

		gtk_tree_model_get( GTK_TREE_MODEL( thumbnailStore ),	&iter,	COL_PATH,	&path,	-1 );

		prevImage	=	path;

		this->setFocusOnItem( thumbnailView,	GTK_TREE_MODEL( thumbnailStore ),	iter );
	}
	
	mw->updateStatusbar( "Scanning for images... Done" );
	
	g_dir_close( dir );
}

//-------------------------------------------------------------------------
// changeImageClicked method
void FileView::changeImageClicked( GtkWidget *widget,	gpointer data )
{
	GtkTreeIter iter;
	
	GList *list	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( thumbnailView ) );
	list	=	g_list_first( list );
	
	gtk_tree_model_get_iter( GTK_TREE_MODEL( thumbnailStore ),	&iter,	(GtkTreePath*)list->data );
	
	g_list_free( list );

	if( widget	==	GTK_WIDGET( prevButton )	||	widget	==	GTK_WIDGET( view ) )
	{
		GtkTreePath *path	=	gtk_tree_model_get_path( GTK_TREE_MODEL( thumbnailStore ),	&iter );
		
		gtk_tree_path_prev( path );
		if( gtk_tree_model_get_iter( GTK_TREE_MODEL( thumbnailStore ),	&iter,	path ) == false )
		{
			gtk_list_store_append( thumbnailStore,	&iter );
			
			gtk_tree_path_free( path );
			
			path	=	gtk_tree_model_get_path( GTK_TREE_MODEL( thumbnailStore ),	&iter );
			
			gtk_tree_path_prev( path );
			gtk_list_store_remove( thumbnailStore,	&iter );
			gtk_tree_model_get_iter( GTK_TREE_MODEL( thumbnailStore ),	&iter,	path );
		}
		gtk_tree_path_free( path );
	}
		
	else
	{
		gtk_tree_model_iter_next( GTK_TREE_MODEL( thumbnailStore ),	&iter );
	
		if( !gtk_list_store_iter_is_valid( thumbnailStore, &iter ) )
			gtk_tree_model_get_iter_first( GTK_TREE_MODEL( thumbnailStore ),	&iter );
	}
	
	gtk_icon_view_unselect_all( GTK_ICON_VIEW( thumbnailView ) );
	this->setFocusOnItem( thumbnailView,	GTK_TREE_MODEL( thumbnailStore ),	iter );
}

//-------------------------------------------------------------------------
// dragStart method
void FileView::dragStart( GtkWidget *widget,	GdkDragContext *context,	gpointer data )
{
	showMenu	=	false;	
}

//-------------------------------------------------------------------------
// dragEnd method
void FileView::dragEnd( GtkWidget *widget,	GdkDragContext *context,	gpointer data )
{
	showMenu	=	true;
}

//-------------------------------------------------------------------------
// dragReceived method
void FileView::dragReceived( GtkWidget *widget, 
			     GdkDragContext *context, 
			     int x, 
			     int y,
			     GtkSelectionData *data,
			     guint info,
			     guint time, 
			     gpointer user_data )		
{
	string path,	paths	=	( char * )data->data;
	string::size_type pos	=	0;
	
	if( pasteData.begin()	!=	pasteData.end() ) 
		pasteData.clear();
		
	if( g_str_has_prefix( paths.c_str(),	"http" ) 	|| 
		g_str_has_prefix( paths.c_str(),	"ftp" ) )
	{
		
		pasteData.push_back( paths );
		
		mw->setPasteData( pasteData );
		
		PasteFunc *p	=	new	PasteFunc( this,	TYPE_DOWNLOAD );
		pthread_create( &pasteThread,	NULL,	&createPasteThread,	p );
		
		return;
	}
	
	paths	+=	"\n";
	
	while( 1 )
	{
		pos	=	paths.find( "\n",	0 );
		
		if( pos	==	string::npos ) 
			break;
		
		path	=	paths.substr( 0,	pos );
		pasteData.push_back( path );
		paths.erase( 0,	pos	+	1 );
	}
	
	mw->setPasteData( pasteData );

	if( context->suggested_action	==	GDK_ACTION_ASK )
	{
		gtk_menu_popup( dnd_menu,	NULL,	NULL,	NULL,	NULL,	3,	0 );
		gtk_widget_show_all( GTK_WIDGET( dnd_menu ) );
		return;
	}
	
	pasteType	=	TYPE_CUT;
	this->pasteClicked( widget,	NULL );
}

//-------------------------------------------------------------------------
// getDragData method
void FileView::getDragData( GtkWidget *widget,
		       				GdkDragContext *context,
		       				GtkSelectionData *data,
		       				guint info,
		       				guint time,
		       				gpointer user_data )
{	
	GtkTreeIter i;
	const char *path;
	
	GList *selections;
	
	if( type	==	ICON_VIEW) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if( type	==	LIST_VIEW) 
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	
	if( selections	==	NULL ) 
		return;
	
	guint len	=	g_list_length( selections );
	char *paths[ len ];
	int it	=	0;
	
	if( pasteData.begin()	!=	pasteData.end() ) 
		pasteData.clear();
	
	GList *list	=	g_list_first( selections );
	
	while( 1 )
	{	
		gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&i, 
								( GtkTreePath* )list->data );
		gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&i,
							COLUMN_PATH,	&path,	-1 );
		
		pasteData.push_back( (char*)path );
		
		paths[ it ]	=	( char* )path;
		it++;
		
		list	=	g_list_next( list );
		
		if( !list ) 
			break;
	}
	
	paths[ it ]	=	NULL;
	
	char *content	=	g_strjoinv( "\n",	paths );
	
    	gtk_selection_data_set( data,	data->target,	8, 
							( guchar* )content,	strlen( content ) );
	
	g_list_free( list );
	g_list_free( selections );
	//TODO: figure out why seg fault here and debugger saying double free. This doesn't need to be free'd?
	//g_free(content);
}

//-------------------------------------------------------------------------
// buttonReleased method
gboolean FileView::buttonReleased( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{
	if( showMenu	&&	button->button == 3 )
	{
		gtk_menu_popup( menu,	NULL,	NULL,	NULL,	NULL,	3,	button->time );

		GList *selections	=	NULL;
	
		if( type	==	ICON_VIEW ) 
			selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
		else if( type	==	LIST_VIEW )
		{
			selection		=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
			selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
		}
		
		if( selections && g_list_length( selections ) == 1 )
		{
			GList *first	=	NULL;	
			first	=	g_list_first( selections );

			GtkTreeIter iter;	
			
			gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&iter,	(GtkTreePath*)first->data );
			
			const char *path;
			
			gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&iter,	COLUMN_PATH,	&path,	-1 );
			
			bool test	=	g_file_test( path,	G_FILE_TEST_IS_DIR ); 
			
			for( vector<GtkWidget*>::iterator itr = alternativeActions.begin();	itr != alternativeActions.end();	itr++ )
				gtk_widget_destroy( (*itr) );
			
			alternativeActions.clear();

			if( test )
				gtk_widget_hide_all( GTK_WIDGET( openWithSubmenu ) ); 
			else
			{
				gtk_widget_show_all( GTK_WIDGET( openWithSubmenu ) );

				const char *mime	=	xdg_mime_get_mime_type_for_file( path );
				string home		=	g_get_home_dir();
				string file		=	home	+	( string )"/.gbrowser/actions";		
				GKeyFile* actions	=	g_key_file_new();

				g_key_file_load_from_file( actions,	file.c_str(),	G_KEY_FILE_NONE,	NULL );

				char **keys	=	g_key_file_get_keys( actions,	mime,	NULL,	NULL );
				
				string def;
					
				if( g_key_file_has_key( actions,	mime,	"Default",	NULL ) )
					def	=	g_key_file_get_string( actions,	mime,	"Default",	NULL );

				if( keys[0] != NULL )
					for( int i = 0;	keys[i] != NULL;	i++ )
					{	
						if( keys[i] == NULL )
							break;

						if( Util::compare( keys[i],	"Default" ) ||
				     		    Util::compare( keys[i],	def ) )
							continue;
					
						GtkWidget *item	=	gtk_menu_item_new_with_label( keys[i] );
					
						gtk_menu_shell_prepend( GTK_MENU_SHELL( gtk_menu_item_get_submenu( openWithSubmenu ) ),	item );
						
						gtk_widget_show( item );
						
						alternativeActions.push_back( item );
						
						openAlternativeCallback.connect( G_OBJECT( item ),	"button-release-event",	this,
													&FileView::openAlternativeClicked,	NULL );
						if( i == 4 )
							break;
					}
			}

			g_list_free( first );
			g_list_free( selections );
		}
		else
			gtk_widget_hide_all( GTK_WIDGET( openWithSubmenu ) );
	}
	
	gesture		=	false;
	
	if( button->button	!=	2 )
		return false;

	double	x,	y;
	x	=	button->x; 
	y	=	button->y;
	
	if( x	<	x_axis	&&
		y	<=	y_axis	+	50	&& 
		y	>=	y_axis	-	50 ) mw->prevClicked( NULL,	NULL );
	
	else if( x	>	x_axis	&& 
			 y	<=	y_axis	+	50	&& 
			 y	>=	y_axis	-	50 ) mw->nextClicked( NULL,	NULL );
	
	else if( y	<	y_axis	&& 
			 x	<=	x_axis	+	50	&& 
			 x	>=	x_axis	-	50 ) mw->parentClicked( NULL,	NULL );
	
	else if( y	>	y_axis	&& 
			 x	<=	x_axis	+	50	&& 
			 x	>=	x_axis	-	50 ) this->scanDirectory( currentDir );
	
	else 
		mw->updateStatusbar( "Unknow mouse gesture received" );

	return false;
}

//-------------------------------------------------------------------------
// dndMenuClicked method
void FileView::dndMenuClicked( GtkWidget *widget,	gpointer data )
{
	if( widget	==	GTK_WIDGET( dnd_move ) )
		this->pasteClicked( widget,	NULL );
	
	else if( widget	==	GTK_WIDGET( dnd_copy ) )
		this->pasteClicked( widget,	NULL );
	
	else if( widget	==	GTK_WIDGET( dnd_symlink ) )
	{
		pasteData	=	checkFileListExist( mw->getPasteData() );
		
		for( list<string>::iterator iter	=	pasteData.begin(); 
									iter	!=	pasteData.end(); 
									iter++ )
		{
			const char *src,	*name;
			src		=	(*iter).c_str();
			name	=	g_path_get_basename( src );
			
			string cmd	=	(string)"ln -sf \""	+	src	+	(string)"\" \""	+	
							currentDir	+	(string)"/"	+	name	+	(string)"\"";
			system( cmd.c_str() );
		}
	}
}

//-------------------------------------------------------------------------
// checkFileListExist method
list<string> FileView::checkFileListExist( list<string> paths )
{
	bool exists	=	true;
	
	for( list<string>::iterator iter	=	paths.begin(); 
								iter	!=	paths.end(); 
								iter++ )
	{
		string path	=	currentDir	+	"/"	+	(*iter);
		
		if( g_file_test( path.c_str(),	G_FILE_TEST_EXISTS ) ) 
			exists	=	Dialog::fileExists( g_path_get_basename( path.c_str() ) );
		
		if( !exists ) 
			iter	=	paths.erase( iter );
		
		exists	=	true;
	}
	return paths;
}

//-------------------------------------------------------------------------
// needsTerminal method
bool FileView::needsTerminal( const string& mime,	string command )
{
	string actionsFile	=	g_get_home_dir()	+	(string)"/.gbrowser/actions";
	
	GKeyFile* actions	=	g_key_file_new();
	g_key_file_load_from_file( actions,	actionsFile.c_str(),	G_KEY_FILE_NONE,	NULL );

	if( command[ command.size()	-	1 ]	==	' ' ) 
		command	=	command.substr( 0,	command.size()	-	1 );
	
	if( !g_key_file_has_key( actions,	mime.c_str(),	command.c_str(),	NULL ) ) 
		return false;

	bool b	=	g_key_file_get_boolean( actions,	mime.c_str(),	command.c_str(),	NULL );

	g_key_file_free( actions );
	
	return b;
}

//-------------------------------------------------------------------------
// setFocusOnItem method
void FileView::setFocusOnItem( GtkWidget *tree_view,	GtkTreeModel *model,	GtkTreeIter tree_iter )
{
	if( tree_view	==	NULL )
		tree_view	=	view;
	if( model	==	NULL )
		model	=	GTK_TREE_MODEL( fileStore );
	
	GtkTreePath *path	=	gtk_tree_model_get_path( model,	&tree_iter );
	
	if( type	!=	LIST_VIEW )
		gtk_icon_view_select_path( GTK_ICON_VIEW( tree_view ),	path );
	else
		gtk_tree_selection_select_path( selection,	path );
	
	if( type == IMAGE_VIEW )
	{
		GtkAdjustment *hadj =	 gtk_scrolled_window_get_hadjustment( GTK_SCROLLED_WINDOW( thumbnailView->parent ) );
		
		step	+=	( hadj->upper / imageTotal );
		
		if(	 step	>	hadj->lower	&&	step	<=	( hadj->upper - ( hadj->page_size/2 ) ) )
		{
			double adj	=	 step	-	( hadj->page_size / 2 );
			gtk_adjustment_set_value( hadj,	adj );
		}
	}
	
	gtk_tree_path_free( path );
}

//-------------------------------------------------------------------------
// handleFile method
void FileView::handleFile( string realName,	string fullPath,	bool setFocus )
{	
	mw->handleFile(	fileStore,	realName,	fullPath,	setFocus,	type,	currentDir );
}

//-------------------------------------------------------------------------
// tabChanged method
void FileView::tabChanged()	const
{
	mw->setViewMode();

	typedef Dp1<Sidebar, void, string > dp1;
	dp1 *func	=	new dp1( Sidebar::getInstance(),	&Sidebar::treeViewSetPath,	currentDir );
	
	Dispatch::getInstance()->addToQueue( func );
}

//-------------------------------------------------------------------------
// imageChanged method
void FileView::imageChanged( GtkIconView *icon_view,	gpointer data )
{
	string newName	=	gtk_entry_get_text( imageNameEntry );
   	string newPath	=	currentDir	+	"/"	+	newName;
	
	GtkTreeIter iter;
	
	GList *list	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( thumbnailView ) );
	list	=	g_list_first( list );
	
	if( !list )
		return;
	
	gtk_tree_model_get_iter( GTK_TREE_MODEL( thumbnailStore ),	&iter,	(GtkTreePath*)list->data );
	
	g_list_free( list );

	if( newName.size() != 0	&&	!Util::compare( newPath,	prevImage ) )
	{
		if( Util::checkWritePermissions( prevImage ) &&
			Util::checkWritePermissions( currentDir ) )
		{
				rename( prevImage.c_str(),	newPath.c_str() );
				gtk_list_store_set( thumbnailStore,	&iter,	COL_PATH,	newPath.c_str(),	
										COL_NAME,	newName.c_str(),
										-1 );
		}
		else if( g_file_test( prevImage.c_str(),	G_FILE_TEST_EXISTS ) )
			mw->updateStatusbar( "You don't have permissions to rename " + prevImage );
	}

	GList	*selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( thumbnailView ) );
	
	if( selections == NULL )
		return;

	gtk_tree_model_get_iter( GTK_TREE_MODEL( thumbnailStore ),	&iter,	(GtkTreePath*)selections->data );

	const char *path,	*name;
	int pos;
	
	gtk_tree_model_get( GTK_TREE_MODEL( thumbnailStore ), &iter,	COL_NAME,	&name,	
									COL_PATH,	&path,	
									COL_POS,	&pos,	-1 );

       	if( imageAngleChanged )
		if( Dialog::confirmDialog( "GBrowser",	
			"Warning!\nImage Angle have been changed.\nDo you want to overwrite original image with current one?" ) )
		{
                	GdkPixbuf *src	=	gtk_image_get_pixbuf( image );

			string format		=	prevImage.substr( prevImage.find_last_of( "." ) + 1 );

			if( g_utf8_validate( format.c_str(),	format.size(),	NULL ) )
				format	=	g_utf8_strdown( format.c_str(),	format.size() );
			else
				format	=	g_ascii_strdown( format.c_str(),	format.size() );

			if( Util::compare( format,	"jpg" ) )
				format	=	"jpeg";

			remove( prevImage.c_str() );

			gdk_pixbuf_save( src, prevImage.c_str(), format.c_str(), NULL,	NULL );

			imageAngleChanged	=	false;
		}

	string status		=	Get::getString( pos ) + "/" + Get::getString( imageTotal );
	
	gtk_entry_set_text( imageNameEntry,	name );
	gtk_statusbar_push( imageNumStatus,	0,	status.c_str() );
	
	gtk_image_set_from_file( image,	path );
	
	g_list_free( selections );

	prevImage	=	path;
}

//-------------------------------------------------------------------------
// imageClicked method
gboolean FileView::imageClicked( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{
	if( button->button == 1 )
	{
		this->changeImageClicked( GTK_WIDGET( nextButton ),	NULL );
		return true;
	}
	else if( button->button == 3 )
	{
		gtk_menu_popup( menu,	NULL,	NULL,	NULL,	NULL,	3,	button->time );
		gtk_widget_show_all( GTK_WIDGET( menu ) );
		return true;
	}
	
	return false;
}

//-------------------------------------------------------------------------
// rotateImageClicked method
void FileView::rotateImageClicked( GtkWidget *widget,	gpointer data )
{
	GdkPixbufRotation angle;

	GdkPixbuf *src,	*buf,	*thumb;

	const char *path;
	
	GtkTreeIter iter;
	
	GList *list	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( thumbnailView ) );
	list	=	g_list_first( list );
	
	gtk_tree_model_get_iter( GTK_TREE_MODEL( thumbnailStore ),	&iter,	(GtkTreePath*)list->data );
	
	g_list_free( list );

	gtk_tree_model_get( GTK_TREE_MODEL( thumbnailStore ),	&iter,	COL_PATH,	&path,	-1 );

	src	=	gtk_image_get_pixbuf( image );

	if( widget	==	GTK_WIDGET( rotateLeft ) )
		angle	=	GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
	else
		angle	=	GDK_PIXBUF_ROTATE_CLOCKWISE;

        buf	=	gdk_pixbuf_rotate_simple( src,	angle );

	thumb	=	gdk_pixbuf_scale_simple( buf,	80,	80,	GDK_INTERP_BILINEAR );

	gtk_list_store_set( thumbnailStore,	&iter,	COL_ICON,	thumb,	-1 );
	gtk_image_set_from_pixbuf( image,	buf );

	imageAngleChanged	=	true;
}

//-------------------------------------------------------------------------
// openWithClicked method
gboolean FileView::openWithClicked( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{
	string action	=	Dialog::createNewFuncDialog( "Open With: ",	"Open With",	"",	"",	false );
	
	if( action.size() <= 0 )
		return false;
	
	GList *selections;
	
	if( type	==	ICON_VIEW ) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if( type	==	LIST_VIEW )
	{
		selection		=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}
	
	if( !selections )
		return false;
	
	GList *first	=	g_list_first( selections );
	
	GtkTreeIter iter;
	
	gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&iter,	(GtkTreePath*)first->data );
	
	const char *path;
	
	gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&iter,	COLUMN_PATH,	&path,	-1 );
	
	string cmd		=	action + (string)" " + path;
	const char* mime	=	xdg_mime_get_mime_type_for_file( path );
	
	GKeyFile* actions	=	g_key_file_new();
	string file			=	g_get_home_dir() + (string)"/.gbrowser/actions";
	
	g_key_file_load_from_file( actions,	file.c_str(),	G_KEY_FILE_NONE,	NULL );
	
	if( action[ action.size() - 1 ]	==	' ' ) 
		action	=	action.substr( 0,	action.size() - 1 );
	
	g_key_file_set_boolean( actions,	mime,	action.c_str(),	FALSE );
	
	ofstream val( file.c_str() );
	char *v	=	g_key_file_to_data( actions,	NULL,	NULL );
	val	<<	v;
	val.close();
	
	g_free( v );
	g_key_file_free( actions );
	
	pid_t pid;
	pid	=	fork();
	
	if( pid	==	0 )
	{
		system( cmd.c_str() );
		exit( EXIT_SUCCESS );
	}
	
	g_list_free( first );
	g_list_free( selections );
	
	return true;
}

//-------------------------------------------------------------------------
// addBookmarkClicked method
gboolean FileView::addBookmarkClicked( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{
	GList *selections;
	GtkTreeIter iter;

	if( type	==	ICON_VIEW ) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if( type	==	LIST_VIEW )
	{
		selection	=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}

	if( !selections )
	{
		Sidebar::getInstance()->addBookmark( currentDir );
		return false;
	}

	GList *sel	=	g_list_first( selections );

	while( sel )
	{
		gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&iter,	(GtkTreePath*)sel->data );

		const char *path;
		gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&iter,	COLUMN_PATH,	&path,	-1 );

		if( g_file_test( path,	G_FILE_TEST_IS_DIR ) )
			Sidebar::getInstance()->addBookmark( path );

		sel	=	g_list_next( sel );
	}	

	g_list_free( sel );
	g_list_free( selections );
	
	return false;
}

//-------------------------------------------------------------------------
// readFamInput method
//NOTE: static member function
void	FileView::readFamInput( gpointer data,	gint fd,	GdkInputCondition cond )
{
	FAMConnection *fam = (FAMConnection *)data;
 
	while( FAMPending( fam ) )
	{
		FAMEvent ev;
		if( FAMNextEvent( fam,	&ev ) != 1 )
		{
			gdk_input_remove( fd );
			FAMClose( fam );
			g_free( fam );
			return;
		}
 
		if( ev.code != FAMDeleted	&&	ev.code != FAMCreated )
			continue;
		
		FileView *fv	=	(FileView*)ev.userdata;
		
		if( ev.code == FAMDeleted )
		{
			list<string> paths;
			string path	=	fv->getPath() + (string)"/" + (string)ev.filename;
			
			paths.push_back( path );
			
			fv->removeEntries( NULL,	paths );
		}
		else if( ev.code == FAMCreated )
		{
			string path	=	fv->getPath() + (string)"/" + (string)ev.filename;
			
			fv->handleFile( ev.filename,	path,	false );
		}
	}
}

gboolean FileView::openAlternativeClicked( GtkWidget *widget,		GdkEventButton *button,	gpointer data )
{
	GtkWidget *label	=	gtk_bin_get_child( GTK_BIN( widget ) );
	
	string action	=	gtk_label_get_text( GTK_LABEL( label ) );
	
	GList *selections;

	if( type	==	ICON_VIEW ) 
		selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( view ) );
	else if( type	==	LIST_VIEW )
	{
		selection		=	gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
		selections	=	gtk_tree_selection_get_selected_rows( selection,	NULL );
	}
	
	if( !selections )
		return false;
	
	GList *first	=	g_list_first( selections );
	
	GtkTreeIter iter;
	
	gtk_tree_model_get_iter( GTK_TREE_MODEL( fileStore ),	&iter,	(GtkTreePath*)first->data );

	const char *path;
	gtk_tree_model_get( GTK_TREE_MODEL( fileStore ),	&iter,	COLUMN_PATH,	&path,	-1 );

	string cmd	=	action + " \"" + (string)path + "\"";
	
	Util::exec( cmd );

	g_list_free( first );
	g_list_free( selections);

	return false;
}
