/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FILEVIEW_H
#define FILEVIEW_H

#include <list>
#include <fam.h>

#include "mainwindow.h"
#include "fileinfo.h"
#include "tab.h"

//icon and list view
enum{
	COLUMN_ICON,
	COLUMN_NAME,
	COLUMN_SIZE,
	COLUMN_PATH,
	COLUMN_IS_DIR,
	COLUMN_OWNER,
	COLUMN_GROUP,
	COLUMN_PERMS,
	COLUMN_ACCESS
};

//image view thumbnail view
enum
{
	COL_ICON,
	COL_NAME,
	COL_PATH,
	COL_POS
};

typedef enum{
	ICON_VIEW,
	LIST_VIEW,
	IMAGE_VIEW
} ViewMode;

static GtkTargetEntry target[] = {
	{ "STRING",	0,	0 },
	{ "text/plain",	0,	0 },
	{ "text/uri-list",	0,	1 },
};

class FileView : public Tab
{
public:
	FileView( std::string path );
	~FileView();

	inline	GtkWidget	*getWidget()	const	{return window;};
	inline	GtkListStore	*getStore()	const	{return fileStore;};
		
	inline	std::string	getPath()	const		{return currentDir;};
		
	inline	std::list<std::string>	getPrevList()	const	{return prevDirs;};
	inline	std::list<std::string>	getNextDirs()	const	{return nextDirs;};
		
	inline 	void	setShowHidden	( bool hidden ){ showHidden = hidden; };
			
	void	buildIconView();
	void	buildListView();
	void	buildImageView();
	void	clearTreeViews();
	void	createSignalHandlers();
	void	tabChanged()	const;
	
	void	setView			( GtkRadioMenuItem *item );
	void	enterFolder		( GtkTreePath *path );
	void	removeEntries		( GtkListStore *store, std::list<std::string> data );
	
	void	scanDirectory		( std::string path );
	void	scanImages		( std::string path );
	void	startScanning		( std::string path );
	void	handlePrev		( std::string path );
	void	handleNext		( std::string path );
	void	handleParent		( std::string path );
	void	openFile			( const std::string& path );
	void	setFocusOnItem	( GtkWidget *tree_view,	GtkTreeModel *model,	GtkTreeIter iter );
	void	handleFile		( std::string realName,	std::string fullPath,	bool setFocus);
		
	std::list<std::string>	checkFileListExist		( std::list<std::string> paths );
	std::string			getDefaultRunAction	( const std::string& mime );
	bool 					needsTerminal			( const std::string& mime,	std::string command );

	//signals
	void 	deleteClicked			( GtkWidget *widget,		gpointer data );	
	void 	pasteClicked			( GtkWidget *widget,		gpointer data );
	void 	cutClicked			( GtkWidget *widget,		gpointer data );
	void 	copyClicked			( GtkWidget *widget,		gpointer data );
	void 	fileInfoClicked			( GtkWidget *widget,		gpointer data );
	void 	renameClicked			( GtkWidget *widget,		gpointer data );
	void 	changeImageClicked	( GtkWidget *widget,		gpointer data );
	void	iconEnterFolder		( GtkWidget *widget,		GtkTreePath *path,	gpointer data )	{ this->enterFolder( path ); };	
	void 	listEnterFolder			( GtkWidget *widget,		GtkTreePath *path,	GtkTreeViewColumn *col,	gpointer data ){ this->enterFolder( path ); };
	void	dragStart				( GtkWidget *widget,		GdkDragContext *context,	gpointer data);
	void	dragEnd				( GtkWidget *widget,		GdkDragContext *context,	gpointer data);
	void	rotateImageClicked		( GtkWidget *widget,		gpointer data );
	void 	dndMenuClicked		( GtkWidget *widget,		gpointer data );
	void	imageChanged		( GtkIconView *icon_view,	gpointer data );
	void	dragReceived			( GtkWidget *widget,		GdkDragContext *context,
						  		int x,
						  		int y,
						  		GtkSelectionData *data,
						  		guint info,
                           			  		guint time,
                           			  		gpointer user_data );
	void 	getDragData			( GtkWidget *widget,		GdkDragContext *context,
	       					  		GtkSelectionData *selection_data,
	       					  		guint info,
	       					  		guint time,
	       					  		gpointer data );
	
	gboolean	openWithClicked		( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
	gboolean 	newFileClicked			( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
	gboolean 	newLinkClicked			( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
	gboolean 	newFolderClicked		( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
	gboolean	addBookmarkClicked	( GtkWidget *widget,		GdkEventButton *button,	gpointer data );	
	gboolean 	buttonReleased		( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
	gboolean 	imageClicked			( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
	gboolean 	showPopupMenu		( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
	gboolean openAlternativeClicked	( GtkWidget *widget,		GdkEventButton *button,	gpointer data );
		
	static	void	*createPasteThread	( void *arg );
	static	void	*createDeleteThread	( void *arg );
	static	void	readFamInput			( gpointer data,	gint fd,	GdkInputCondition cond );
	
protected:
	GtkListStore		*fileStore,		*thumbnailStore;
	GtkMenu 			*menu,			*dnd_menu;
	GtkMenuItem		*copyItem,		*cutItem,			*pasteItem,	*deleteItem,	*newFolder, 
					*newFile,			*newLink,			*properties,	*newTabItem,	*renameItem, 
					*dnd_move,		*dnd_copy,		*dnd_symlink,	*openItem,	*bookmarkItem,
					*newSubmenu,	*actionsSubmenu,	*openWithSubmenu;
	GtkTreeSelection	*selection;
	GtkWidget 		*window,			*view,			*thumbnailView;
	GtkButton 		*prevButton,		*nextButton,		*rotateLeft,	*rotateRight;
	GtkImage 		*image;
	GtkStatusbar		*imageNumStatus;
	GtkEntry			*imageNameEntry;
	FAMConnection	*fam;
	FAMRequest		*fr;

	pthread_t pasteThread;

	MainWindow *mw;

	bool	showHidden,	allowNext,	allowPrev,	gesture,	imageAngleChanged,	showMenu;
	int	pathColumn,	targetSize,	pasteType,	imageCount,	imageTotal,	tag;
	double 	x_axis,		y_axis,	step;

	ViewMode type;

	std::string	currentDir,	prevImage;
	
	std::list<std::string>	pasteData,	deleteData,	prevDirs,	nextDirs;
	std::vector<GtkWidget*> alternativeActions;
		
	Func2<FileView, void, GtkWidget *>	deleteCallback,		pasteCallback,		cutCallback,
									copyCallback,			fileInfoCallback,	renameCallback,	
									changeImageCallback,	dndCallback,		rotateImageCallback;
	Func2<FileView, void, GtkIconView *>	imageSelectionChanged;
	Func3<FileView, gboolean, GtkWidget *, GdkEventButton *>	showMenuCallback,		imageClickedCallback,	buttonReleaseCallback,	
														addBookmarkCallback,	newFileCallback, 		newLinkCallback,
														newFolderCallback,		openWithCallback,		openAlternativeCallback;
	Func3<FileView, void, GtkWidget *, GtkTreePath *>	iconEnterCallback;
	Func3<FileView, void, GtkWidget *, GdkDragContext *>	dragStartCallback,	dragEndCallback;
	Func4<FileView, void, GtkWidget *, GtkTreePath *, GtkTreeViewColumn * >		listEnterCallback;
	Func6<FileView, void, GtkWidget *, GdkDragContext *, GtkSelectionData *, guint, guint>	getDataCallback;
	Func8<FileView, void, GtkWidget *, GdkDragContext *, int, int, GtkSelectionData *, guint, guint>	dragReceiveCallback;
};
#else
class FileView;
#endif
