/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "get.h"

using namespace std;

string Get::getString( int i )
{
	char	f[ sizeof( i ) ];
	sprintf( f,	"%d",	i );
	return f;
}

string Get::getTime( int seconds )
{
	int	minutes	=	0,	
		hours	=	0,	
		days		=	0;
	
	while( seconds	>	59)
	{
		seconds	=	seconds	-	60;
		minutes++;
	}
	
	while( minutes	>	59 )
	{
		minutes	=	minutes	-	60;
		hours++;
	}
	
	while( hours	>	23 )
	{
		hours	=	hours	-	24;
		days++;
	}
	
	string output	=	"";
	
	if( days	>	0 ) 
		output	+=	getString( days )		+	( string )" Days, ";
	
	if( hours	>	0 ) 
		output 	+=	getString( hours )		+	( string )" Hours, ";
		
	if( minutes	>	0 ) 
		output	+=	getString( minutes )	+	( string )" Minutes, ";
		
	if( seconds	>	0 ) 
		output	+=	getString( seconds )	+	( string )" Seconds";
		
	if( output[ output.size()	-	1]	==	',' ) 
		output	=	output.substr( 0,	output.size()	-	1 );
		
	return output;
}

const char *Get::getSize( double size )
{
	char	d[1024],	*c	=	"b";

	if( size	>=	1024 )
	{
		size	=	size	/	1024;
		c	=	"Kb";
	}
	
	if( size	>=	1024)
	{
		size	=	size	/	1024;
		c	=	"Mb";
	}
	
	if( size	>=	1024)
	{
		size	=	size	/	1024;
		c	=	"Gb";
	}

	sprintf( d,	"%.2f %s",	size,	c );
	
	return d;
}

const char *Get::getUtf8( const string& text )
{
	return g_filename_to_utf8( text.c_str(),	-1,	NULL,	NULL,	NULL );
}

int Get::getDirTotal( const string& path )
{
	list<string> m	=	Util::recursiveScan( path,	false );
	
	return m.size();
}

double Get::getFileSize( const string& path )
{
	ifstream inFile( path.c_str(),	ios::in );
	
	inFile.seekg( ios::beg,	ios::end );
	
	double	inputSize	=	inFile.tellg();
	
	inFile.close();
	
	return inputSize;
}

double Get::getFolderSize( const string& path )
{
	double	total;
	list<string> files	=	Util::recursiveScan( path,	false );
	list<string>::iterator i;

	string	in;

	for( i	=	files.begin();	i	!=	files.end();	i++ )
	{
		in	=	( *i );
		
		ifstream	infile( in.c_str() );
		
		infile.seekg( 0,	ios::end );
		total	+=	infile.tellg();
		
		infile.close();
	}

	files.clear();
	
	return total;
}

//-------------------------------------------------------------------------
// getExifData method
#if ENABLE_EXIF
list<string> Get::getExifData( const string& file )
{
	list<string> entries;
	
	string entry	=	"";
	
	const char *name;
	
	unsigned int tag,	iter;
	
	ExifLoader *loader = exif_loader_new();
	exif_loader_write_file( loader,	file.c_str() );
	ExifData *data	=	exif_loader_get_data( loader );
	exif_loader_unref( loader );

	if( !data || ! (data->data || data->size ||
		data->ifd[EXIF_IFD_0]->count ||
		data->ifd[EXIF_IFD_1]->count ||
		data->ifd[EXIF_IFD_EXIF]->count ||
		data->ifd[EXIF_IFD_GPS]->count ||
		data->ifd[EXIF_IFD_INTEROPERABILITY]->count) )
			return entries;

	for( tag	=	0; 
		 tag	<	0xffff; 
		 tag++ )
	{
		name	=	exif_tag_get_title( (ExifTag)tag );

		if( !name )
			continue;
		
		entry	=	name;

		for( iter	=	0; 
			 iter	<	EXIF_IFD_COUNT; 
			 iter++ )
		{
				ExifEntry	*e	=	exif_content_get_entry( data->ifd[iter], (ExifTag)tag );
			
				if( e )
				{
					char v[1024];
					string content	=	exif_entry_get_value( e, v, 73 );
					string tmp	=	exif_tag_get_title( e->tag ) + (string)": ";

					entries.push_back(  tmp + content );
				}
		}
	}
	
	list<string>::iterator it;
	
	return entries;
}
#endif

//-------------------------------------------------------------------------
// getUser method
string Get::getUser( guint uid )
{
	struct passwd *pw;
		
	pw	=	getpwuid( uid );
		
	string user;
		
	if( pw )
		user	=	pw->pw_name;
	else
		user	=	getString( (int)uid );
		
	return user;
}

//-------------------------------------------------------------------------
// getGroup method
string Get::getGroup( guint gid )
{
	struct group *grp;
		
	grp	=	getgrgid( gid );
		
	string group;
		
	if( grp )
		group	=	grp->gr_name;
	else
		group	=	getString( (int)gid );	
		
	return group;
}

//-------------------------------------------------------------------------
// getParentMime method
string Get::getParentMime( const string& mime )
{
	string tmp	=	mime;
		
	tmp	=	tmp.substr( 0,	tmp.find( "/" ) );
	
	return tmp;
}

//-------------------------------------------------------------------------
// getParentMime method
string Get::getSubMime( const string& mime )
{
	string tmp	=	mime;
	
	tmp	=	tmp.substr( tmp.find( "/" ) + 1,	tmp.npos );
	
	return tmp;
}
