/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "pasteFunc.h"
#include <curl/curl.h>

using namespace std;

gboolean createTimer(void *arg)
{
	PasteFunc *p = (PasteFunc *)arg;
	p->handleProgressStatus();
}

void finishTimer(void *arg)
{
	PasteFunc *p =(PasteFunc *)arg;
	p->finishPaste();
}

PasteFunc::PasteFunc(FileView *arg, int t)
{
	fv = arg;
	mw = MainWindow::getInstance();
	
	type = t;
	
	currentDir = fv->getPath();
	
	items = 0;
	inputTotal = 0;
	outputTotal = 0;
	p_total = 0;
	lastSize = 0;

	hBox = gtk_hbox_new(false, 0);
	vbox = mw->getMainBox();
	bar = gtk_progress_bar_new();
	cancelButton = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	gtk_button_set_use_stock(GTK_BUTTON(cancelButton), true);

	gtk_box_pack_end(GTK_BOX(vbox), hBox, false, false, 0);
	gtk_box_pack_start(GTK_BOX(hBox), bar, true, true, 0);
	gtk_box_pack_end(GTK_BOX(hBox), cancelButton, false, false, 0);
	gtk_widget_show(hBox);
	gtk_widget_show(bar);
	gtk_widget_show(cancelButton);

	if(type != TYPE_DOWNLOAD)
		id = g_timeout_add_full(G_PRIORITY_HIGH_IDLE, 200, &createTimer, (void*)this, finishTimer);
}

PasteFunc::~PasteFunc()
{
	gdk_threads_enter();
	g_source_remove(id); //remove timeout handler
	gtk_widget_ref(hBox);
	gtk_container_remove(GTK_CONTAINER(vbox), hBox);
	gtk_widget_destroy(hBox); //try to destroy the box
	gdk_threads_leave();
}

void PasteFunc::pasteThread()
{	
	pasteDataList = mw->getPasteData();
	
	if(type == TYPE_DOWNLOAD){
		this->downloadFunc((*pasteDataList.begin()));
		return;
	}
	
	for(list<string>::iterator it = pasteDataList.begin(); it != pasteDataList.end(); it++){
		string t = (*it);
		
		if(type == TYPE_DELETE) p_total += Get::getDirTotal(t);
		
		if(g_file_test(t.c_str(), G_FILE_TEST_IS_DIR)) 
			 inputTotal += Get::getFolderSize(t);
		else inputTotal += Get::getFileSize(t);
	}

	for(list<string>::iterator it = pasteDataList.begin(); it != pasteDataList.end(); it++){
		string t = (*it);
		string p = currentDir + (string)"/" + g_path_get_basename(t.c_str());
		
		this->pasteFunc(t, currentDir);
	}

	delete this;
	pthread_exit( NULL );
}

void PasteFunc::pasteFunc(string inputPath, string outputPath)
{
	string command;
	p_input = inputPath;
	
	if(type == TYPE_COPY) command = "cp -a ";
	else command = "mv ";
		
	command += "\"" + inputPath + (string)"\" \"" + outputPath + (string)"\"";
	system(command.c_str());
}

void PasteFunc::deleteThread()
{
	list<string>::iterator it,	iter;
	
	pasteDataList	=	mw->getDeleteData();
	p_total		=	pasteDataList.size();

	for(it	=	pasteDataList.begin(); 
	    it	!=	pasteDataList.end(); 
	    it++)
	{
		p_input	=	(*it);
		
		gdk_threads_enter();
		mw->updateStatusbar( "Removing " + p_input );
		gdk_threads_leave();
		
		if( g_file_test( p_input.c_str(),	G_FILE_TEST_IS_DIR ) ) 
		{
			list<string> files	=	Util::recursiveScan( p_input,	false );
			list<string> dirs	=	Util::recursiveScan( p_input,	true );
		
			for( iter	=	files.begin(); 
			     iter	!=	files.end(); 
			     iter++ )
			{
				if( Util::checkWritePermissions( (*iter) ) ) 
					remove( (*iter).c_str() );
				else
				{
					gdk_threads_enter();
					mw->updateStatusbar( "Could not delete: " + 
							     (*iter) + 
							     (string)" because of permissions" );
					gdk_threads_leave();
				}
			}
			gdk_threads_enter();
			mw->updateStatusbar( "Cleaning out empty directories" );
			gdk_threads_leave();
		
			for( iter	=	dirs.begin(); 
			     iter	!=	dirs.end(); 
			     iter++ )
			{
				if( Util::checkWritePermissions((*iter) ) )
					rmdir( (*iter).c_str() );
				else
				{
					gdk_threads_enter();
					mw->updateStatusbar( "Could not delete: " + 
							     (*iter) + 
							     (string)" because of permissions" );
					gdk_threads_leave();
				}
			}
		}
		
		g_remove( p_input.c_str() );
		items++;
	}
	
	delete this;
	pthread_exit( NULL );
}

void PasteFunc::handleProgressStatus()
{
	string func, output, filename;

	filename = g_path_get_basename(p_input.c_str());
	output = currentDir + (string)"/" + filename;
	
	if(type == TYPE_COPY)func = "Copying: ";
	else if(type == TYPE_DELETE) func = "Removing: ";
	else func = "Moving: ";
	
	double size;
	if(type != TYPE_DELETE){
		if(g_file_test(output.c_str(), G_FILE_TEST_IS_DIR)) 
			size = Get::getFolderSize(output);
		else size = Get::getFileSize(output);
			
		outputTotal += size;
		if(size >= lastSize)outputTotal -= lastSize;
		lastSize = size;
	}
	
	string totalItems = Get::getString(p_total);
	string it = Get::getString(items);
	string totalOut = Get::getSize(outputTotal);
	totalSize = Get::getSize(inputTotal);
	
	double percentage;
	string pStatus;
	
	pStatus = func + filename;
	if(type != TYPE_DELETE){ 
		pStatus += " " + totalOut + (string)"/" + totalSize;
		percentage = outputTotal/inputTotal;
	}
	else percentage = items/p_total;

	if(percentage >= 0 && percentage <= 1.0)
		this->updateProgressbar(percentage, pStatus);
	lastFilename = filename;
}

void PasteFunc::finishPaste()
{
	string status;
	
	if(type == TYPE_DELETE) 
		status	=	"Remove Finished";
	else if(type == TYPE_COPY)
		status	=	"File Copied";
	else if(type == TYPE_CUT)
		status	=	"File Moved";
	else 
		status	=	"File Downloaded";

	mw->updateStatusbar( status );
	this->updateProgressbar( 0,	"" );
}

void PasteFunc::updateProgressbar(double percentage, string text)
{		
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(bar), text.c_str());
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(bar), percentage);
}

size_t writeFunc(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	return fwrite(ptr, size, nmemb, stream);
}
 
size_t readFunc(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	return fread(ptr, size, nmemb, stream);
}

const char *outFile;
GTimer *timer;
double last, lastTime;

int progress(GtkWidget *bar,
                     double t, /* dltotal */
                     double d, /* dlnow */
                     double ultotal,
                     double ulnow)
{
	gulong mSeconds;
	
	double time = g_timer_elapsed(timer, &mSeconds);
	double dl_rate = (d - last) / (time - lastTime);
	double eta = (t - d) / dl_rate;
	
	string speed = Get::getSize((int)dl_rate) + (string)"/s";
	string size = Get::getSize(t);
	
	string text = (string)"Downloading: " + Get::getUtf8(outFile) + Get::getSize(d) + (string)"/" + 
		size + (string)" Speed: " + speed + (string)"\nETA: " + Get::getTime((int)eta);
	
	gdk_threads_enter();
	gtk_progress_set_value(GTK_PROGRESS(bar), d*100.0/t);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(bar), text.c_str());
	gdk_threads_leave();
	
	last = d;
	lastTime = time;
	
	return 0;
}

void PasteFunc::downloadFunc(string url)
{
	CURL *conn = curl_easy_init();
	
	outFile = g_path_get_basename(url.c_str());
	string outPath = currentDir + "/" + string(outFile);
	FILE *out = fopen(outPath.c_str(), "w");
	
	last = 0;
	lastTime = 0;
	
	timer = g_timer_new();
	g_timer_start(timer);
	
	curl_easy_setopt(conn, CURLOPT_URL, url.c_str());
	curl_easy_setopt(conn, CURLOPT_WRITEDATA, out);
	curl_easy_setopt(conn, CURLOPT_WRITEFUNCTION, writeFunc);
	curl_easy_setopt(conn, CURLOPT_READFUNCTION, readFunc);
	curl_easy_setopt(conn, CURLOPT_NOPROGRESS, FALSE);
	curl_easy_setopt(conn, CURLOPT_PROGRESSFUNCTION, progress);
	curl_easy_setopt(conn, CURLOPT_PROGRESSDATA, bar);
	
	curl_easy_perform(conn);
	
	fclose(out);
	
	curl_easy_cleanup(conn);
	
	gdk_threads_enter();
	this->finishPaste();
	gdk_threads_leave();
	
	delete this;
	pthread_exit( NULL );
}
