/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "util.h"

using namespace std;

//-------------------------------------------------------------------------
// recursiveScan method
list<string> Util::recursiveScan( const string& path,	bool directories )
{
	list<string>	dirs,	results;

	dirs.push_back( path );

	for( list<string>::iterator	it	=	dirs.begin();	it	!= dirs.end();	it++ )
	{
		string	currPath	=	( *it );

		GDir	*dir	=	g_dir_open( currPath.c_str(),	0,	NULL );

		const char	*file	=	g_dir_read_name( dir );
		
		g_dir_rewind( dir );
		
		while( file	!=	NULL )
		{
			file	=	g_dir_read_name( dir );
			if( file	==	NULL )
				break;

			string	fullPath	=	currPath	+	"/"	+	file;

			if( !directories	&&	g_file_test( fullPath.c_str(),	G_FILE_TEST_IS_DIR ) ) 
				dirs.push_back( fullPath );
			
			else if( !directories	&&	!g_file_test( fullPath.c_str(),	G_FILE_TEST_IS_DIR ) ) 
				results.push_back( fullPath );
			
			else if( directories	&&	g_file_test( fullPath.c_str(),	G_FILE_TEST_IS_DIR ) )
			{
				dirs.push_back( fullPath );
				results.push_front( fullPath );
			}
			
		}
		
		g_dir_close( dir );
	}
	return results;
}

//-------------------------------------------------------------------------
// sortFunc method
int Util::sortFunc( GtkTreeModel *model,	GtkTreeIter *a,	GtkTreeIter *b,	gpointer data )
{
	gboolean dirA	=	true,	
		 dirB	=	true;
  	char 	*nameA,	*nameB;
  	int 	ret;
  
	if( (int)data	!=	1 )
	{
  		gtk_tree_model_get( model,	a,
		      			COLUMN_IS_DIR,	&dirA,
		      			COLUMN_NAME,	&nameA,
		      			-1 );

  		gtk_tree_model_get( model,	b,
		      			COLUMN_IS_DIR,	&dirB,
		      			COLUMN_NAME,	&nameB,
		      			-1 );
	}

  	if( !dirA	&&	dirB ) 
		ret	=	1;
  	else if( dirA	&&	!dirB ) 
		ret	=	-1;
  	else 
		ret	=	g_utf8_collate( nameA,	nameB );

  	g_free( nameA );
  	g_free( nameB );

  	return ret;
}

//-------------------------------------------------------------------------
// sortFuncOnlyDirs method
int Util::sortFuncOnlyDirs( GtkTreeModel *model,	GtkTreeIter *a,	GtkTreeIter *b,	gpointer data )
{
  	char 	*nameA,	*nameB;
  	int 	ret;

	gtk_tree_model_get( model,	a,
		      	 1,	&nameA,
		      	-1 );

	gtk_tree_model_get( model,	b,
		      	 1,	&nameB,
		      	-1 );

	ret	=	g_utf8_collate( nameA,	nameB );

  	g_free( nameA );
  	g_free( nameB );

  	return ret;
}

//-------------------------------------------------------------------------
// executableExists method
bool Util::executableExists(const string& executable)
{
	if( g_file_test( executable.c_str(),	(GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_EXECUTABLE ) ) )
		return true;
	
	const char *exe	=	g_find_program_in_path( executable.c_str() );
	
	if( exe	==	NULL )
		return false;
		
	return true;
}

//-------------------------------------------------------------------------
// checkReadPermissions method
bool Util::checkReadPermissions( const string& file )
{
 	int rval	=	access( file.c_str(),	R_OK );
	
 	if( rval	==	0 ) 
		return true;
	
 	else 
		return false;
}

//-------------------------------------------------------------------------
// checkWritePermissions method
bool Util::checkWritePermissions( const string& file )
{
 	int	rval = access (file.c_str(), W_OK);
	
 	if( rval	==	0 ) 
		return true;
	
	else 
		return false;
}

//-------------------------------------------------------------------------
// compare method
bool Util::compare( const string& a, const string& b )
{
	if( a.length()	!=	b.length() ) 
		return false;

	for( int	i	=	0;	i	!=	a.length();	i++ )
		if( a[ i ]	!=	b[ i ] ) 
			return false;
		
	return true;
}

//-------------------------------------------------------------------------
// exec method
void Util::exec( const string& command )
{
	pid_t pid;
	pid	=	fork();
	
	if( pid	==	0 )
	{
		system( command.c_str() );
		exit( EXIT_SUCCESS );
	}
}

